/**********************************************************************
** Copyright (C) 2002 Edgar Hindemith.  All rights reserved.
**
** email: ed@dom.de
**
** This file is part of the qpPhoto picture editor.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/
#ifndef DRAWVIEWINCLUDED
#define DRAWVIEWINCLUDED

#include <qwidget.h>
#include <qscrollview.h>
#include <qvaluelist.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qstring.h>
#include <qevent.h>
#include <qarray.h>
#include <qregion.h>
#include <qxml.h>

class DrawView : public QScrollView
{
    Q_OBJECT
public:
    DrawView( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~DrawView();
    void init(int w = -1, int h = -1, const QColor bg = white, const QColor fg = black);
    void init(QString fileName, const QColor bg = white, const QColor fg = black);
    void setBrushImage(int size);
    void setBrushImage(const QPixmap& mask);
    void setBrushImage(const char** mask);
    void setBrushImage(const QPixmap& image, const QPixmap& mask);
    void setErazorImage(int size);
    void setErazorImage(const QPixmap& mask);
    void setErazorImage(const QPixmap& image, const QPixmap& mask);
    QColor getColorAt(int x, int y);
    bool isSelectionEmpty(){ return selection.isEmpty(); };
    bool isClipEmpty(){ return clipBits.isNull(); };
    QPoint getPos(){ return QPoint(mouse_x, mouse_y); };
    QPoint getTrackerPos(){ return QPoint(trackerOrigin_x, trackerOrigin_y); };
    bool isMousePressed(){ return mouse_pressed; };
    int imageWidth(){ return imgWidth; };
    int imageHeight(){ return imgHeight; };
    const char* getFormat(){ return imageFormat; };
    QPixmap getPixmap(){ return graphic; };
    bool edited(){ return changed; };
    void setEdited(bool e){ changed = e; };
    int getDepth(){ return depth; };
    bool isGray(){ return gray; };
    int getZoomIndex(){ return zoomIndex; };
    void enableUndo(bool enUndo = false);
    bool isUndoEnabled(){ return undoEnabled; };

    QFont font;
    QBrush brush;
    QPen pen;
    QColor fgColor;
    QColor bgColor;

    QPixmap brushSprite;
    QImage brushImage;
    QImage brushMask;
    QPixmap erazorSprite;
    QImage erazorImage;
    QImage erazorMask;
    int brushWidth;
    int brushHeight;
    int erazorWidth;
    int erazorHeight;
    bool brushSolid;
    bool erazorSolid;

    int currentTool;
    int currentShape;
    int currentFill;
    int currentSelType;
    int currentSelOp;
    int currentLineCap;
    int currentRO;
    int currentROC;

    enum rasterOps
    {
        roSTD,
        roAND,
        roOR,
        roCP,
        roADD,
        roSUB,
        roLIGHT,
        roDARK,
        roAVG,
        roMASK,
        roMIX
    };

    enum ropCondition
    {
        rocALWAYS,
        rocIFGT,
        rocIFLT
    };

    enum pointType
    {
        ptStart,
        ptMiddle,
        ptEnd0,
        ptEnd1,
        ptEnd2,
        ptEnd3,
        ptEnd4
    };

    enum directions
    {
        dirNull,
        dir12,
        dir1_30,
        dir3,
        dir4_30,
        dir6,
        dir7_30,
        dir9,
        dir10_30
    };

    enum trackertype
    {
        trLine,
        trHLine,
        trVLine,
        trPolyLine,
        trRect,
        trCircle,
        trEllipse,
        trMove
    };

    enum zoomStates
    {
        zsCanZoom,
        zsZoomInOnly,
        zsZoomOutOnly
    };

    enum undoStates
    {
        usCantDo,
        usCanUndo,
        usCanRedo
    };

    enum undoTypes
    {
        utImage,
        utRegion
    };

signals:
    void   mouseMoved();
    void   mousePressed();
    void   mouseReleased();
    void   colorPicked(QColor);
    void   selectionChanged();
    void   zoomStateChanged(int);
    void   undoStateChanged(int);
    void   clipStateChanged();
    void   colorStateChanged(int, bool);

public slots:
    void   resizeContents(int w, int h);
    void   editUndo();
    void   editRedo();
    void   editCut();
    void   editCopy();
    void   editPaste();
    void   hideMask(bool h);
    void   invertMask();
    void   clearMask();
    void   zoomIn();
    void   zoomOut();
    void   zoom(int level);
    void   resizeImage();
    void   hFlip();
    void   vFlip();
    void   rot180();
    void   rot90();
    void   darken();
    void   lighten();
    void   blur();
    void   sharpen();
    void   emboss();
    void   convertToGray();
    void   convertToMono();
    void   convertToIndexed();
    void   convertToRGB();

protected slots:
    void   timeoutUpdate();

protected:
    QColor pixelOp(QRgb src, QRgb dst, int rop);
    QColor pixelColor(int x, int y, QImage& dst, QImage& src, QImage& mask, int rop = roAND, int ropc = rocALWAYS, bool useMask=true);
    void   blit(int x, int y, int pt = ptStart);
    void   drawPoint(int x, int y);
    void   drawEndPoint(int x, int y);
    void   drawLineTo(int x, int y);
    void   drawTracker(int x, int y, int type = trRect, bool draw = true);
    void   startTracker(int x, int y);
    void   track(int x, int y);
    void   endTracker(int x, int y);
    void   drawLine(QPainter& painter, int x1, int y1, int x2, int y2);
    void   doDrawTool(int x, int y);
    void   fill(QPainter& painter, int start = 0, int end = 0);
    void   pickColor(int x, int y);
    void   setSelection(int x, int y, int w, int h);
    void   findArea(int x, int y, QColor c, QRegion& r, int op = 1);
    void   moveSelection(int dx, int dy);
    void   drawSelection(QPainter *p);
    void   paste(int x, int y);
    //void   drawIt( QPainter *p );
    //void   paintEvent( QPaintEvent * );
    void   setUndo();
    void   init(QImage img);
    void   scale();
    void   setImage(QImage img);
    void   contentsMouseMoveEvent(QMouseEvent* mev);
    void   contentsMousePressEvent(QMouseEvent* mev);
    void   contentsMouseReleaseEvent(QMouseEvent* mev);
    void   drawContents(QPainter* p, int clipx, int clipy, int clipw, int cliph);
    void   filter(int f_matrix[3][3], int fact = 0, int bias = 0);
    void   filterFu(QImage *newimage, QRegion sel, int f_matrix[3][3], int fact = 0, int bias = 0);

    QPixmap graphic;

private:
    int viewX;
    int viewY;
    int imgWidth;
    int imgHeight;
    int scaledWidth;
    int scaledHeight;
    int mouse_x;
    int mouse_y;
    int trackerOrigin_x;
    int trackerOrigin_y;
    int last_x;
    int last_y;
    int direction;
    int last_direction;
    int zoomIndex;
    int depth;
    bool isScaled;
    bool changed;
    bool pasting;
    bool gray;
    const char* imageFormat;
    QRect trackerMoveRect;

    QImage image;
    QPixmap pmScaled;
    QPixmap pmUndo;
    QPixmap clipBits;

    int undoState;
    int undoType;
    bool selection_hidden;
    bool selection_user_hidden;
    bool mouse_pressed;
    bool scToggle;
    bool undoEnabled;
    QRegion selection;
    QRegion clipRegion;
    QRegion undoRegion;
    QValueList<QPoint> points;
};

#endif // DRAWVIEWINCLUDED
